//
// $Id: errors.cc 2 2007-12-30 09:48:00Z hat $
//
#include "errors.h"
#include <iostream>

std::string sProgname;
std::string sFilename;

static void report_message(int line, const char *mtype, const std::string& msg)
{
	bool needSep = false;

	if(sProgname.length() > 0) {
		std::cerr << sProgname; needSep = true;
	}

	if(sFilename.length() > 0) {
		if (needSep) { std::cerr << ", "; }
		std::cerr << "in \"" << sFilename << "\"";
		needSep = true;
	}

	if(line > 0) {
		if (needSep) { std::cerr << " "; }
		std::cerr << "at line " << line;
		needSep = true;
	}

	if(needSep) { std::cerr << ", "; }
	std::cerr << mtype << ": " << msg << "\n";
}

/** Report a warning to the user
 *  @param line: Line number of the warning. Use 0 if line number is unknown
 *  @param msg: Text of the warning message
 */
void report_warning(int line, const std::string& msg)
{
	report_message(line, "Warning", msg);
}

/** Report an error to the user
 *  @param line: Line number of the error. Use 0 if line number is unknown
 *  @param msg: Text of the error message
 */
void report_error(int line, const std::string& msg)
{
	report_message(line, "Error", msg);
}

/** Set program and filename for reported errors
 *  @param sP: Name of the program
 *  @param sF: Name of the input file
 */
void set_filename(const std::string& sP, const std::string& sF)
{
	sProgname = sP; sFilename = sF;
}

