//
// $Id: generic-ast.cc 2 2007-12-30 09:48:00Z hat $
//
#include "generic-ast.h"
#include "errors.h"

/* {{{ IdentifierLine */
IdentifierLine::IdentifierLine()
	: sIden(""), iLine(0) { }
IdentifierLine::IdentifierLine(const string& sId, int iL)
	: sIden(sId), iLine(iL) { }

IdentifierLine::IdentifierLine(const IdentifierLine& cil)
	: sIden(cil.sIden), iLine(cil.iLine) { }

IdentifierLine& IdentifierLine::operator=(const IdentifierLine& cil)
{
	if(this != &cil) {
		sIden = cil.sIden; iLine = cil.iLine;
	}
	return *this;
}
IdentifierLine::~IdentifierLine() { }

string IdentifierLine::get_iden() const { return sIden; }
int IdentifierLine::get_line() const { return iLine; }

/* }}}
 *
 * {{{ BaseField */
BaseField::BaseField(int il): iLine(il) { }
BaseField::~BaseField() { }

void BaseField::set_line(int il) { iLine = il; }
int BaseField::get_line() const { return iLine; }

/* }}}
 * {{{ Field */
Field::Field(const string& sN, int iL): BaseField(iL), sName(sN) { }
Field::~Field() { }

/* }}}
 * {{{ FieldKey */
FieldKey::FieldKey(const string& sN, int iL): Field(sN, iL) { }
FieldKey::~FieldKey() { }

/* }}}
 * {{{ FieldIdenValue */
FieldIdenValue::FieldIdenValue(const string& key, int key_line,
                               const string& iden_val, int iden_line)
	: Field(key, key_line),
	  sIdenValue(iden_val), iIdenValueLine(iden_line) { }
FieldIdenValue::~FieldIdenValue() { }

/* }}}
 * {{{ FieldIdentifierSet */
FieldIdentifierSet::FieldIdentifierSet(const string& sN, int iL,
				       const IdentifierSet& cidset)
	: Field(sN, iL), cIdentifiers(cidset) { }
FieldIdentifierSet::~FieldIdentifierSet() { }

/* }}}
 * {{{ FieldUnsigned */
FieldUnsigned::FieldUnsigned(const string& sN, int iL, UINT32 val, int lin)
	: Field(sN, iL), value(val), line(lin) { }
FieldUnsigned::~FieldUnsigned() { }

/* }}}
 * {{{ FieldSigned */
FieldSigned::FieldSigned(const string& sN, int iL, INT32 val, int lin)
	: Field(sN, iL), value(val), line(lin) { }
FieldSigned::~FieldSigned() { }

/* }}}
 * {{{ FieldString */
FieldString::FieldString(const string& sN, int iL, const TextString& cT)
	: Field(sN, iL), cText(cT) { }
FieldString::~FieldString() { }

/* }}}
 * {{{ FieldBytes */
FieldBytes::FieldBytes(const string& sN, int iL, const ByteSequence& cBS)
	: Field(sN, iL), cBytes(cBS) { }
FieldBytes::~FieldBytes() { }

/* }}}
 * {{{ FieldLabels, LabelLine
 */
LabelLine::LabelLine(): sLabel(""), line(0) { }
LabelLine::LabelLine(const string& sL, int lin): sLabel(sL), line(lin) { }
LabelLine::LabelLine(const LabelLine& cl): sLabel(cl.sLabel), line(cl.line) { }
LabelLine& LabelLine::operator=(const LabelLine& cl)
{
    if(this != &cl) {
        sLabel = cl.sLabel; line = cl.line;
    }
    return *this;
}
LabelLine::~LabelLine() { }

/* FieldLabels */
FieldLabels::FieldLabels(const string& key, int key_line,
                         const list<LabelLine>& cL)
        : Field(key, key_line), cLabels(cL) { }
FieldLabels::~FieldLabels() { }


/* {{{ GenericNode
 */
GenericNode::GenericNode(): BaseField() { }
GenericNode::~GenericNode()
{
	while(!cFields.empty()) {
		delete cFields.back();
		cFields.pop_back();
	}
}

void GenericNode::add_field(BaseField* pBF)
{
	/* Silently skip pointers to 0 */
	if(pBF)
		cFields.push_back(pBF);
}

void GenericNode::set_name(const string& sN) { sName = sN; }
void GenericNode::set_label(const string& sL, int line)
{
	sLabel = sL;
	if(sL == "exit") {
		report_error(line, "Label \"exit\" should not be used");
	}
}
/* }}} */
