//
// $Id: getopt.cc 2 2007-12-30 09:48:00Z hat $
//
#include "getopt.h"
#include <iostream>

int optind = 1, opterr = 1;
static char *optkar = 0;
char *optarg;

void reset_getopt()
{
    optind = 1; optkar = 0; opterr = 1;
}

int getopt(int argc, char *argv[], const char *options)
{
    const char *opt_ptr;

    if(optind >= argc) return -1;

    if(!optkar || !*optkar) {
        optkar = argv[optind];
        ++optind;

        if(*optkar != '-') {
            --optind; return -1;
        }
        ++optkar;
        if(!*optkar) {
            --optind; return -1; // "-" recognized
        }
    }

    opt_ptr = options;
    while(*opt_ptr) {
        if(*opt_ptr == ':' || *opt_ptr != *optkar) {
            ++opt_ptr; continue;
        }
        // *opt_ptr == *optkar   => option recognized
        if(*(opt_ptr + 1) != ':') {     // normal option
            optarg = 0; ++optkar; return *opt_ptr;
        }

        optarg = optkar + 1;
        if(*optarg) {
            optkar = 0; return *opt_ptr;
        }

        if(optind < argc) {
            optarg = argv[optind]; ++optind;
            optkar = 0; return *opt_ptr;
        }

        if(opterr) {
            std::cerr << "Missing argument of option '-" << *opt_ptr << "'\n";
        }
        optind = argc; return '?';
    }

    if(opterr) {
        std::cerr << "Unrecognized option '-" << *optkar << "'\n";
    }
    optind = argc; return '?';
}

#ifdef TEST_GETOPT

#include <cassert>

void test1()
{
    char *argv[] = { "prg", "-x", 0 };
    int i;

    reset_getopt();

    i = getopt(2, argv, "sa:");
    assert(i == '?');
}

void test2a()
{
    char *argv[] = { "prg", "-s", "val", 0 };
    int i;

    reset_getopt();

    i = getopt(3, argv, "sa:");
    assert(i == 's');
    i = getopt(3, argv, "sa:");
    assert(i == -1);
    assert(optind == 2);
}

void test2b()
{
    char *argv[] = { "prg", "-s", "val", 0 };
    int i;

    reset_getopt();

    i = getopt(3, argv, "a:s");
    assert(i == 's');
    i = getopt(3, argv, "a:s");
    assert(i == -1);
    assert(optind == 2);
}

void test3()
{
    char *argv[] = { "prg", "-s", 0 };
    int i;

    reset_getopt();

    i = getopt(2, argv, "a:s");
    assert(i == 's');
    i = getopt(2, argv, "a:s");
    assert(i == -1);
    assert(optind == 2);
}

void test4a()
{
    char *argv[] = { "prg", "-a", "arg", 0 };
    int i;

    reset_getopt();

    i = getopt(3, argv, "a:s");
    assert(i == 'a');
    assert(optarg[0] == 'a' && optarg[1] == 'r' && optarg[2] == 'g');
    i = getopt(3, argv, "a:s");
    assert(i == -1);
    assert(optind == 3);
}

void test4b()
{
    char *argv[] = { "prg", "-a", "arg", 0 };
    int i;

    reset_getopt();

    i = getopt(3, argv, "sa:");
    assert(i == 'a');
    assert(optarg[0] == 'a' && optarg[1] == 'r' && optarg[2] == 'g');
    i = getopt(3, argv, "sa:");
    assert(i == -1);
    assert(optind == 3);
}

void test5()
{
    char *argv[] = { "prg", "-sa", "arg", 0 };
    int i;

    reset_getopt();

    i = getopt(3, argv, "sa:");
    assert(i == 's');
    i = getopt(3, argv, "sa:");
    assert(i == 'a');
    assert(optarg[0] == 'a' && optarg[1] == 'r' && optarg[2] == 'g');
    i = getopt(3, argv, "sa:");
    assert(i == -1);
    assert(optind == 3);
}

void test6()
{
    char *argv[] = { "prg", "-saarg", "-s", "val", 0 };
    int i;

    reset_getopt();

    i = getopt(4, argv, "sa:");
    assert(i == 's');
    i = getopt(4, argv, "sa:");
    assert(i == 'a');
    assert(optarg[0] == 'a' && optarg[1] == 'r' && optarg[2] == 'g');
    i = getopt(4, argv, "sa:");
    assert(i == 's');
    i = getopt(4, argv, "sa:");
    assert(i == -1);
    assert(optind == 3);
}

void test7()
{
    char *argv[] = { "prg", "-a", 0 };
    int i;

    reset_getopt();

    i = getopt(2, argv, "sa:");
    assert(i == '?');
}

int main()
{
    test1();
    test2a(); test2b();
    test3();
    test4a(); test4b();
    test5();
    test6();
    test7();
    return 0;
}

#endif
