//
// $Id: main.cc 2 2007-12-30 09:48:00Z hat $
//
#include "yyfile.h"
#include "errors.h"
//#include "textstring.h"
#include "node.h"
#include <cstdio>
#include <cassert>
#include <iostream>
#include "getopt.h"

static const char *const usage =
    "Usage: grlenc [options] grl-file\n"
    "with options:\n"
    "  -h       (This) online help\n"
    "  -f FILE  Identifier tables file\n"
    "  -s       Turn on scanner debugging\n"
    "  -p       Turn on parser debugging\n";

/** Parse the arguments at the command line
 * @return: 0=ok, run encoder, 1=ok, exit, 2=failed, exit
 */
static int parse_cmdline(int argc, char *argv[],
    const char **pGrlFilename, bool *debug_scanner, bool *debug_parser,
    const char **pTablesFilename)
{
    int val;

    assert(pGrlFilename); assert(debug_scanner); assert(debug_parser);
    assert(pTablesFilename);
    *pGrlFilename = 0; *debug_scanner = false; *debug_parser = false;
    *pTablesFilename = 0;

    /* initialize the report_error() code */
    set_filename(std::string("grlenc"), std::string(""));

    opterr = 0; /* no error reported by getopt */
    val = getopt(argc, argv, "hf:sp");
    while(val != -1) {
        switch(val) {
        case '?': report_error(-1, "Unrecognized option, try 'grlenc -h'");
                  return 2;
        case 'h': std::cout << usage; return 1;
        case 's': *debug_scanner = true; break;
        case 'p': *debug_parser = true; break;
        case 'f': *pTablesFilename = optarg; break;
        default: assert(0);
        }
        val = getopt(argc, argv, "hf:sp");
    }
    
    if(optind == argc) {
        report_error(-1, "Missing grl-file");
        return 2;
    }
    if(optind + 1 == argc) {
        *pGrlFilename = argv[optind];
        return 0;
    }

    report_error(-1, "Too many arguments, 'grlenc -h'");
    return 2;
}


int main(int argc, char *argv[])
{
    FILE *fp;
    int result;
    MainNode *pMainNode;

    const char *pGrlFname, *pTablesFilename;
    bool debug_scanner, debug_parser;

    result = parse_cmdline(argc, argv,
                           &pGrlFname, &debug_scanner, &debug_parser,
                           &pTablesFilename);

    if(result != 0) {
        if(result == 1) exit(0);
        if(result == 2) exit(1);
        assert(0); exit(2);
    }


    /* if available, load table file */
    if(pTablesFilename) {
        set_filename(std::string("grlenc"), std::string(pTablesFilename));
        fp = fopen(pTablesFilename, "ra");
        if(!fp) {
            report_error(-1, "Cannot open tables file");
            exit(1);
        }
        lex_init(fp, debug_scanner);
        pMainNode = parse_file(debug_parser);
        if(pMainNode)           // Silently delete any data read
            delete pMainNode;   // note: tables are saved in 'cTables'.
        else {
            report_error(-1, "Errors reported in table file, aborting");
            exit(1);
        }
    }

    /* load grl file */
    set_filename(std::string("grlenc"), std::string(pGrlFname));
    fp = fopen(pGrlFname, "ra");
    if(!fp) {
        report_error(-1, "Cannot open grl file");
        exit(1);
    }

    lex_init(fp, debug_scanner);
    pMainNode = parse_file(debug_parser);
    fclose(fp);

    if(pMainNode) {
        pMainNode->generate_nfo(stdout);
        delete pMainNode;
        return 0;   // return ok to caller
    } else {
        return 1;   // return error to caller
    }
}

