
%{
#include "yyfile.h"
#include "errors.h"
#include "generic-ast.h"
#include "node.h"
#include "table.h"

GenericNode *pMainFields;
%}

%token COMMA COLON CURLYOPEN CURLYCLOSE SQOPEN SQCLOSE
%token UNSIGNED SIGNED IDENTIFIER STRING EQUAL PLUS
%token BROPEN BRCLOSE
%token TABLEKW TABLENAMEKW TABLESIZEKW TABLEVALUEKW TABLETYPEKW

%start file

%%

file : top_node
       { if($1.pBField)
             pMainFields->add_field($1.pBField);
       }
     | file COMMA top_node
       { if($3.pBField)
             pMainFields->add_field($3.pBField);
       }
     ;

top_node : labeled_node
           { $$.pBField = $1.pBField; }
         | table_node
           { $$.pBField = 0;}
         ;

/* {{{ labeled_node
*/

labeled_node : IDENTIFIER COLON node
               { $3.pGNode->set_label(*($1.pString), $1.line);
                 delete $1.pString;
                 $$.pBField = decode_node($3.pGNode); }
             | node
               { $$.pBField = decode_node($1.pGNode); }
             ;

node : IDENTIFIER CURLYOPEN field_list CURLYCLOSE
       { $$.pGNode = $3.pGNode;
         $$.pGNode->set_name(*($1.pString)); delete $1.pString;
         $$.pGNode->set_line($1.line); }
     | IDENTIFIER CURLYOPEN CURLYCLOSE
       { $$.pGNode = new GenericNode;
         $$.pGNode->set_name(*($1.pString)); delete $1.pString;
         $$.pGNode->set_line($1.line); }
     ;

field_list : field
             { $$.pGNode = new GenericNode;
               $$.pGNode->add_field($1.pBField); }
           | field_list COMMA field
             { $$.pGNode = $1.pGNode;
               $$.pGNode->add_field($3.pBField); }
           ;


field   : IDENTIFIER
          { $$.pBField = new FieldKey(*($1.pString), $1.line);
            delete $1.pString; }
        | IDENTIFIER EQUAL IDENTIFIER
          { $$.pBField = new FieldIdenValue(*($1.pString), $1.line,
                                           *($3.pString), $3.line);
            delete $1.pString; delete $3.pString; }
        | IDENTIFIER EQUAL set_list
          { $$.pBField = new FieldIdentifierSet(*($1.pString), $1.line,
                                               *($3.pIdenSet));
            delete $1.pString; delete $3.pIdenSet; }
        | IDENTIFIER EQUAL UNSIGNED
          { $$.pBField = new FieldUnsigned(*($1.pString), $1.line,
                                           $3.uint32, $3.line);
            delete $1.pString; }
        | IDENTIFIER EQUAL SIGNED
          { $$.pBField = new FieldSigned(*($1.pString), $1.line,
                                         $3.int32, $3.line);
            delete $1.pString; }
        | IDENTIFIER EQUAL string_list
          { $$.pBField = new FieldString(*($1.pString), $1.line,
                                        *($3.pTextString));
            delete $1.pString; delete $3.pTextString; }
        | IDENTIFIER EQUAL byte_sequence
          { $$.pBField = new FieldBytes(*($1.pString), $1.line, *($3.pBytes));
            delete $1.pString; delete $3.pBytes; }
        | IDENTIFIER EQUAL identifier_sequence
          { $$.pBField = new FieldLabels(*($1.pString), $1.line,
	  				 *($3.pLabelLines));
            delete $1.pString; delete $3.pLabelLines; }
        | labeled_node
          { $$.pBField = $1.pBField; }
        ;

set_list : IDENTIFIER PLUS IDENTIFIER
           { $$.pIdenSet = new IdentifierSet;
             $$.pIdenSet->insert(IdentifierLine(*($1.pString), $1.line));
             $$.pIdenSet->insert(IdentifierLine(*($3.pString), $3.line));
             delete $1.pString; delete $3.pString; }
         | set_list PLUS IDENTIFIER
           { $$.pIdenSet = $1.pIdenSet;
             $$.pIdenSet->insert(IdentifierLine(*($3.pString), $3.line));
             delete $3.pString; }
         ;

string_list : STRING
              { $$.pTextString = $1.pTextString; }
            | string_list STRING
              { $$.pTextString = new TextString(*($1.pTextString)
                                                + *($2.pTextString));
                delete $1.pTextString; delete $2.pTextString; }
            ;


byte_sequence : SQOPEN byte_list SQCLOSE
                { $$.pBytes = $2.pBytes;
                  $$.pBytes->set_line($1.line); }
              ;
/* Empty byte_sequence is not allowed, as it is ambigious with an empty
 * identifier_sequence
 *
 *              | SQOPEN SQCLOSE
 *                { $$.pBytes = new ByteSequence;
 *                  $$.pBytes->set_line($1.line); }
 */

byte_list : UNSIGNED
            { $$.pBytes = new ByteSequence;
              if($1.uint32 > 255)
                report_error($1.line, "Value to large for a byte, truncating");
              $$.pBytes->add_byte($1.uint32 & 0xff); }
          | byte_list COMMA UNSIGNED
            { $$.pBytes = $1.pBytes;
              if($3.uint32 > 255)
                report_error($3.line, "Value to large for a byte, truncating");
              $$.pBytes->add_byte($3.uint32 & 0xff); }
          ;

identifier_sequence : SQOPEN iden_list SQCLOSE
                      { $$.pLabelLines = $2.pLabelLines; }
                    ;

iden_list : IDENTIFIER
            { $$.pLabelLines = new LabelLineList;
	      $$.pLabelLines->push_back(LabelLine(*($1.pString), $1.line));
	      delete $1.pString; }
          | iden_list COMMA IDENTIFIER
	    { $$.pLabelLines = $1.pLabelLines;
	      $$.pLabelLines->push_back(LabelLine(*($3.pString), $3.line));
	      delete $3.pString; }
	  ;


/* }}}
** {{{ Constant tables
*/

table_node : TABLEKW IDENTIFIER CURLYOPEN table_fields CURLYCLOSE
             { $4.pTable->set_name(*($2.pString)); delete $2.pString;
               cTables.add_table(*($4.pTable)); delete $4.pTable;
             }
           ;

table_fields : BROPEN entries BRCLOSE
               { $$.pTable = new Table;
                 $2.pTableEntry->set_line($1.line);
                 $$.pTable->add_entry(*($2.pTableEntry));
                 delete $2.pTableEntry; }
             | table_fields COMMA BROPEN entries BRCLOSE
               { $$.pTable = $1.pTable;
                 $4.pTableEntry->set_line($3.line);
                 $$.pTable->add_entry(*($4.pTableEntry));
                 delete $4.pTableEntry; }
             ;

entries : TABLENAMEKW EQUAL IDENTIFIER
          { $$.pTableEntry = new TableEntry;
            $$.pTableEntry->set_name(*($3.pString));
            delete $3.pString; }
        | TABLEVALUEKW EQUAL UNSIGNED
          { $$.pTableEntry = new TableEntry;
            $$.pTableEntry->set_value($3.uint32); }
        | TABLESIZEKW EQUAL UNSIGNED
          { $$.pTableEntry = new TableEntry;
            $$.pTableEntry->set_size($3.uint32); }
        | TABLETYPEKW EQUAL UNSIGNED
          { $$.pTableEntry = new TableEntry;
            $$.pTableEntry->set_type($3.uint32); }
        | entries COMMA TABLENAMEKW EQUAL IDENTIFIER
          { $$.pTableEntry = $1.pTableEntry;
            $$.pTableEntry->set_name(*($5.pString));
            delete $5.pString; }
        | entries COMMA TABLEVALUEKW EQUAL UNSIGNED
          { $$.pTableEntry = $1.pTableEntry;
            $$.pTableEntry->set_value($5.uint32); }
        | entries COMMA TABLESIZEKW EQUAL UNSIGNED
          { $$.pTableEntry = $1.pTableEntry;
            $$.pTableEntry->set_size($5.uint32); }
        | entries COMMA TABLETYPEKW EQUAL UNSIGNED
          { $$.pTableEntry = $1.pTableEntry;
            $$.pTableEntry->set_type($5.uint32); }
        ;

      ;

/* }}} */
%%

/* {{{ Support routines */

void yyerror(const char *msg)
{
	extern int lineno;

	report_error(lineno, msg);
}

MainNode *parse_file(int debug)
{
	int result;

	pMainFields = new GenericNode;

	yydebug = debug;
	result = yyparse();
	if(result) {	// error occurred
		delete pMainFields;
		return 0;
	}
	pMainFields->set_name("main-node");
	return decode_mainnode(pMainFields);
}

/* }}} */
