/*
** $Id$
*/

%{
#include "yyfile.h"
#include "parser.h"
#include <cstdlib>
#include <string>
#include "errors.h"
#include "textstring.h"

int lineno = 1;		//< Line number of the line being read
TextString text_string;
%}

%x INSTRING INCOMMENT

%%

"table"     { yylval.line = lineno; return TABLEKW; }
"_name"     { yylval.line = lineno; return TABLENAMEKW; }
"_value"    { yylval.line = lineno; return TABLEVALUEKW; }
"_size"     { yylval.line = lineno; return TABLESIZEKW; }
"_type"     { yylval.line = lineno; return TABLETYPEKW; }

[A-Za-z] {
	yylval.line = lineno; yylval.pString = new std::string(yytext);
	return IDENTIFIER;
}
[A-Za-z][-A-Za-z0-9_]*[A-Za-z0-9_] {
	yylval.line = lineno; yylval.pString = new std::string(yytext);
	return IDENTIFIER;
}

"0" {
	yylval.line = lineno; yylval.uint32 = 0; return UNSIGNED;
}
[1-9][0-9]* {
	yylval.line = lineno; yylval.uint32 = strtoul(yytext, 0, 10);
        return UNSIGNED;
}
"0x"[0-9A-Fa-f]+ {
	yylval.line = lineno; yylval.uint32 = strtoul(yytext, 0, 16);
        return UNSIGNED;
}
'.' {
	yylval.line = lineno; yylval.uint32 = yytext[1]; return UNSIGNED;
}

"-"[1-9][0-9]* {
	yylval.line = lineno; yylval.int32 = strtol(yytext, 0, 10);
        return SIGNED;
}
"-0x"[0-9A-Fa-f]+ {
	yylval.line = lineno; yylval.int32 = strtol(yytext, 0, 16);
        return SIGNED;
}

"("	{ yylval.line = lineno; return BROPEN; }
")"	{ yylval.line = lineno; return BRCLOSE; }
"{"	{ yylval.line = lineno; return CURLYOPEN; }
"}"	{ yylval.line = lineno; return CURLYCLOSE; }
"["	{ yylval.line = lineno; return SQOPEN; }
"]"	{ yylval.line = lineno; return SQCLOSE; }
":"	{ yylval.line = lineno; return COLON; }
","	{ yylval.line = lineno; return COMMA; }
"+"	{ yylval.line = lineno; return PLUS; }

"="	{ yylval.line = lineno; return EQUAL; }

"//"		{ BEGIN(INCOMMENT); }
<INCOMMENT>.	;
<INCOMMENT>\n	{ BEGIN(INITIAL); lineno++; }

" "	;
\t	;
\n	{ lineno++; }

\"		{ text_string.clear(); BEGIN(INSTRING); }
<INSTRING>"%{"[a-z]+"}"	{
                  text_string.add_escape_text(yytext); }
<INSTRING>"%{0x"[0-9A-Fa-f][0-9A-Fa-f]?"}" {
		  text_string.add_escape_byte(yytext); }
<INSTRING>\n	{ report_error(lineno, "Unterminated string");
		  BEGIN(INITIAL);
                  yylval.line = lineno;
		  yylval.pTextString = new TextString(text_string); 
		  lineno++; return STRING; }
<INSTRING>[^"]	{ text_string.add_character(yytext); }
<INSTRING>\"	{ BEGIN(INITIAL);
                  yylval.line = lineno;
		  yylval.pTextString = new TextString(text_string);
                  return STRING; }

.	{ char buffer[256];
          if (yytext[0] >= ' ' && yytext[0] <= '~')
          	sprintf(buffer, "Unrecognized character '%s' (value %d)",
		        yytext, yytext[0]);
	  else
          	sprintf(buffer, "Unrecognized character (value %d)",
		        yytext[0]);
          report_error(lineno, buffer); }

%%

/** Initialization of scanner/parser
*/
void lex_init(FILE *fp, int debug)
{
    if(fp) yyrestart(fp);
    BEGIN(INITIAL);
    lineno = 1;
    yy_flex_debug = debug;
}

