//
// $Id: table.cc 2 2007-12-30 09:48:00Z hat $
//
#include <cassert>
#include "table.h"
#include "errors.h"
//#include <iostream>

/* {{{ TableEntry
 */

TableEntry::TableEntry() { reset(); }
TableEntry::TableEntry(const TableEntry& te)
{
    iLine = te.iLine;
    bName = te.bName; bValue = te.bValue;
    bSize = te.bSize; bType = te.bType;
    sName = te.sName; cValue = te.cValue;
    cSize = te.cSize; cType = te.cType;
}
TableEntry& TableEntry::operator=(const TableEntry& te)
{
    if(this != &te) {
        iLine = te.iLine;
        bName = te.bName; bValue = te.bValue;
        bSize = te.bSize; bType = te.bType;
        sName = te.sName; cValue = te.cValue;
        cSize = te.cSize; cType = te.cType;
    }
    return *this;
}

TableEntry::~TableEntry() { }

void TableEntry::reset()
{
    iLine = 0;
    bName = false;
    bValue = false;
    bSize = false;
    bType = false;
}

int TableEntry::get_line() const { return iLine; }
string TableEntry::get_name() const { assert(bName); return sName; }
UINT32 TableEntry::get_value() const { assert(bValue); return cValue; }
UINT32 TableEntry::get_size() const { assert(bSize); return cSize; }
UINT32 TableEntry::get_type() const { assert(bType); return cType; }

bool TableEntry::has_name() const { return bName; }
bool TableEntry::has_value() const { return bValue; }
bool TableEntry::has_size() const { return bSize; }
bool TableEntry::has_type() const { return bType; }

void TableEntry::set_line(int lin) { iLine = lin; }

void TableEntry::set_name(const string& s)
{
    sName = s; bName = true;
}

void TableEntry::set_value(UINT32 val)
{
    cValue = val; bValue = true;
}

void TableEntry::set_size(UINT32 val)
{
    cSize = val; bSize = true;
}

void TableEntry::set_type(UINT32 val)
{
    cType = val; bType = true;
}

/* }}}
 * {{{ Table
 */
Table::Table() { sName = ""; cEntries.clear(); }
Table::Table(const Table& t): sName(t.sName), cEntries(t.cEntries) { }
Table& Table::operator=(const Table& t)
{
    if(this != &t) {
        sName = t.sName;
        cEntries = t.cEntries;
    }
    return *this;
}
Table::~Table() { }

void Table::set_name(const string& s) { sName = s; }
string Table::get_name() const { return sName; }

void Table::add_entry(const TableEntry& te)
{
    if(!te.has_name()) {
        report_error(te.get_line(), "Missing _name field");
        return;
    }

    cEntries[te.get_name()] = te;
}

const TableEntry *Table::get_entry(const string& s) const
{
    map<string, TableEntry>::const_iterator iter;

    iter = cEntries.find(s);
    if(iter == cEntries.end()) return 0;
    return &(*iter).second;
}

/* }}}
 * {{{ Singleton: Tables
 */
Tables::Tables() { }
Tables::~Tables() { }

void Tables::add_table(const Table& t)
{
    //std::cout << "Adding table: " << t.get_name() << "\n";
    cTables[t.get_name()] = t;
}

const Table *Tables::get_table(const string& sName) const
{
    map<string, Table>::const_iterator iter;

    iter = cTables.find(sName);
    if(iter == cTables.end()) return &cDummy;
    return &(*iter).second;
}

Tables cTables;

