/*
 * $Id: textstring.h 2 2007-12-30 09:48:00Z hat $
 */
#ifndef H_TEXTSTRING
#define H_TEXTSTRING

#include <list>
#include <string>
#include <ostream>
#include "types.h"
#include "codehelper.h"

struct DataValue
{
	char *text;
	UINT32 value;
};

class TextFragment;
class TextString;
TextString operator+(const TextString& t1, const TextString& t2);
std::ostream& operator<<(std::ostream& os, const TextString& ts);
bool operator==(const TextFragment&, const TextFragment&);

class TextFragment
{
	friend class TextString;
	friend TextString operator+(const TextString& t1,
	                             const TextString& t2);
	friend std::ostream& operator<<(std::ostream& os,
	                                const TextString& ts);
        friend bool operator==(const TextFragment&, const TextFragment&);
public:
	TextFragment(int ftype, const std::string& st);
	TextFragment(const TextFragment&);
	TextFragment& operator=(const TextFragment&);
	~TextFragment();
protected:
	int frag_type;	//< FRAGTYPE_* value
	std::string sText; //< Text of the fragment
};
#define FRAGTYPE_CHARS    0
#define FRAGTYPE_ESC_TEXT 1
#define FRAGTYPE_ESC_BYTE 2

bool operator==(const TextString&, const TextString&);

class TextString
{
	friend TextString operator+(const TextString& t1,
	                             const TextString& t2);
	friend std::ostream& operator<<(std::ostream& os,
	                                const TextString& ts);
        friend bool operator==(const TextString&, const TextString&);
public:
	TextString();
	TextString(const string&);
	TextString(const TextString&);
	TextString& operator=(const TextString&);
	~TextString();

	std::string convert_to_string();

	void add_text(CodeHelper& ch, const DataValue *pDValues);
	bool empty() const;

	void clear();
	void add_escape_text(const string& sText);
	void add_escape_byte(const string& sText);
	void add_character(const string& sText);
        int find_esc_text(const string& sText, int start=0);
protected:
	std::list<TextFragment> sFragments;

	UINT32 decode_esc_text(const std::string& sText,
	                       const DataValue *pDValues);
};

#endif
